Magnitude subclass: #Grade
	instanceVariableNames: 'gradeLetter gradeSign gradeValue '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Assignment1'!
Grade comment:
'Class Grade
There are no subclassResponsibility messages.  Everything should work fine unless additionaly functionality is added. 
The purpose of the class is to model a grade object which contains a grade letter, a sign (+,-,none), and a point value on a 4.0 scale.

The main pitfall of the grade object is that A+ and A have the same point value.  This makes for some special casing in the comparison messages, and leaves a lot of room for mistakes to be looked over.

Instance Variables:

gradeLetter	<Grade>	Letter of the Grade, used mainly in ascii number format for calculations/changes
gradeSign	<Grade>	a symbol, either #plus, #minus, or #none
gradeValue	<Grade>	point value of the grade, 4.0 scale'!


!Grade methodsFor: 'accessing'!

letter

	" Return the letter value of the grade "

	^Character value: gradeLetter!

modifier

	" Return the sign portion of the grade "

	^gradeSign!

points
	" Return the point value of the grade "

	^gradeValue! !

!Grade methodsFor: 'comparing'!

< anotherGrade
	" Less than comparison "
	" Special Cased for A+ < A  or A < A+ "

	( (self gradeValue = 4) & (self gradeSign = #plus) 
		& (anotherGrade gradeValue = 4) & (anotherGrade gradeSign = #none) )
			ifTrue: [ ^false ]
			ifFalse: [ ( (self gradeValue = 4) & (self gradeSign = #none) 
						& (anotherGrade gradeValue = 4) & (anotherGrade gradeSign = #plus) )
							ifTrue: [ ^true ]
							ifFalse: [ ^self gradeValue < anotherGrade gradeValue ]
					]!

<= anotherGrade
	" less than or equal to comparison "
	" Special Cased for A+ <= A "

	( (self gradeValue = 4) & (self gradeSign = #plus) 
		& (anotherGrade gradeValue = 4) & (anotherGrade gradeSign = #none) )
			ifTrue: [ ^false ]
			ifFalse: [ ^self gradeValue <= anotherGrade gradeValue ]!

= anotherGrade
	" Equal to comparison "
	" Special Cased for A+ = A  or A = A+ "

	( (self gradeValue = 4) & (self gradeSign = #plus) 
		& (anotherGrade gradeValue = 4) & (anotherGrade gradeSign = #none) )
			ifTrue: [ ^false ]
			ifFalse: [ ( (self gradeValue = 4) & (self gradeSign = #none) 
						& (anotherGrade gradeValue = 4) & (anotherGrade gradeSign = #plus) )
							ifTrue: [ ^false ]
							ifFalse: [ ^self gradeValue = anotherGrade gradeValue ]
					]!

> anotherGrade
	" Greater than comparison "
	" Special Cased for A+ > A "

	( (self gradeValue = 4) & (self gradeSign = #plus) 
		& (anotherGrade gradeValue = 4) & (anotherGrade gradeSign = #none) )
			ifTrue: [ ^true ]
			ifFalse: [ ^self gradeValue > anotherGrade gradeValue ]!

>= anotherGrade
	" Greater than or equal to comparison "
	" Special cased for A >= A+ "

	( (self gradeValue = 4) & (self gradeSign = #none) 
		& (anotherGrade gradeValue = 4) & (anotherGrade gradeSign = #plus) )
			ifTrue: [ ^false ]
			ifFalse: [ ^self gradeValue >= anotherGrade gradeValue ]!

between: min and: max
	" Evaluate comparison of whether given grade is between to other grades "

	^(self gradeValue between: max gradeValue and: min gradeValue) 
		| (self gradeValue between: min gradeValue and: max gradeValue)!

max: anotherGrade
	" Evaluate which grade value is greater "

	( anotherGrade gradeValue > self gradeValue )
		ifTrue: [ ^anotherGrade ]
		ifFalse: [ ^self ].!

min: anotherGrade
	" Evaluate which grade value is smaller "

	( anotherGrade gradeValue < self gradeValue )
		ifTrue: [ ^anotherGrade ]
		ifFalse: [ ^self ].!

~= anotherGrade
	" Not Equal to comparison "
	" Special cased for A+ ~= A or A ~= A+ "

	( (self gradeValue = 4) & (self gradeSign = #plus) 
		& (anotherGrade gradeValue = 4) & (anotherGrade gradeSign = #none) )
			ifTrue: [ ^true ]
			ifFalse: [ ( (self gradeValue = 4) & (self gradeSign = #none) 
						& (anotherGrade gradeValue = 4) & (anotherGrade gradeSign = #plus) )
							ifTrue: [ ^true ]
							ifFalse: [ ^self gradeValue ~= anotherGrade gradeValue ]
					]! !

!Grade methodsFor: 'printing'!

printOn: aStream
	" Print the Grade Object "

		" Print Letter "
	aStream nextPut: self gradeLetter.

		" Print Sign "
	( gradeSign = #minus )
		ifTrue: [ aStream nextPut: $- ]
		ifFalse: [ ( gradeSign = #plus )
					ifTrue: [ aStream nextPut: $+ ]
				].! !

!Grade methodsFor: 'arithmetic'!

decrement
	" Decrement the grade by one full letter.  If it is an 'F', do nothing.  Using Ascii, be sure a 'D' is decreased to 			an 'F', not an 'E' "

	( gradeLetter < 68 )
		ifTrue: [ gradeLetter := gradeLetter + 1.
				gradeValue := gradeValue - 1. ]
		ifFalse: [( gradeLetter = 68 )
					ifTrue: [ gradeLetter := 70.
							gradeValue := gradeValue - 1. ]
				]!

increment
	" Increment the grade by one full letter.  If it is an 'A', don't do anything.  Using Ascii, an 'F' has to be sure to 		be made into a 'D', not an 'E' "

	( gradeLetter = 70 )
		ifTrue: [ gradeLetter := gradeLetter - 2.
				gradeValue := gradeValue +1. ]
		ifFalse: [( gradeLetter > 65 )
					ifTrue: [ gradeLetter := gradeLetter - 1.
							gradeValue := gradeValue + 1. ]
				]! !

!Grade methodsFor: 'initialization'!

letter: letterNew modifier: signNew
	"Initialize the Grade letter and its sign"

	gradeLetter := letterNew.
	gradeSign := signNew.
	
	( gradeLetter < 69 )
		ifTrue: [ gradeValue := 69 - gradeLetter ]
		ifFalse: [ gradeValue := 0 ].

	( gradeSign = #plus )
		ifTrue: [ (gradeLetter = 65)
					ifFalse: [gradeValue := gradeValue + 0.3]
				].
	( gradeSign = #minus )
		ifTrue: [ gradeValue := gradeValue - 0.3 ].


	^self! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Grade class
	instanceVariableNames: ''!


!Grade class methodsFor: 'instance creation'!

letter: gradeLetter modifier: gradeSign
	" Check input data, convert and store as integer "

	( gradeLetter asInteger < 71 & gradeLetter asInteger > 64 )
		ifTrue: [ ^(self new) letter: gradeLetter asInteger modifier: gradeSign ]!

new: grade
	" Friendly Init "

	| templetter tempmodifier |

	templetter := grade at: 1.

	( grade size > 1 )
		ifTrue: [ tempmodifier := grade at: 2.
				( tempmodifier = $- )
					ifTrue: [ tempmodifier := #minus ]
					ifFalse: [( tempmodifier = $+ )
								ifTrue: [tempmodifier := #plus ]
							]
			    ]
		ifFalse: [ tempmodifier := #none ].

	^self letter: templetter modifier: tempmodifier! !

