Magnitude subclass: #Grade

	instanceVariableNames: 'letter modifier points '

	classVariableNames: ''

	poolDictionaries: ''

	category: 'Assignment1'!





!Grade methodsFor: 'get'!



letter

	"Return the latter portion of the grade, making sure to convert it 	back to a character before doing so."



	^Character value: letter!



modifier

	"Return the sign portion of the grade, if there is one."



	^modifier!



points

	"Return the associated point value for the grade."



	| tempPoint tempPoints |

	( letter < 69 )

	"Thsi simply calculates the letter's point value based off the ascii 	conversion."

	ifTrue: [ tempPoints := 69 - letter ]

	ifFalse: [ tempPoints := 0 ].



	"Now, we must add, subtract, or do nothing depending on the sign 	part of the grade."

	( modifier == #plus )

	ifTrue: [ tempPoints := tempPoints + 0.3 ].

	( modifier == #minus )

	ifTrue: [ tempPoints := tempPoints - 0.3 ].



	^tempPoints! !



!Grade methodsFor: 'comparisons'!



< otherGrade

	"Determine whether it's less than the other grade."



	^self points < otherGrade points!



<= otherGrade

	"Determine whether it's less than or equal to the other grade."



	^self points <= otherGrade points!



= otherGrade

	"Determine whether it's equal to the other grade."



	^self points = otherGrade points!



> otherGrade

	"Determine whether it's greater than the other grade."



	^self points > otherGrade points!



>= otherGrade

	"Determine whether it's greater than or equal to the other grade."



	^self points >= otherGrade points!



between: min and: max

	"Determine whether the passed grade is between two particular 	grades."



	^(self points between: max points and: min points) | (self points 	  between: min points and: max points)!



max: otherGrade

	"Determine which of the two grades is greater."



	( otherGrade points > self points )

	ifTrue: [ ^otherGrade ]

	ifFalse: [ ^self ].!



min: otherGrade

	"Determine which of the two grades is smaller."



	( otherGrade points < self points )

	ifTrue: [ ^otherGrade ]

	ifFalse: [ ^self ].!



~= otherGrade

	"Determine whether it's not equal to the other grade."



	^self points ~= otherGrade points! !



!Grade methodsFor: 'printing'!



printOn: aStream

	"Used to print out the contents of a grade object."

	

	"Print the actual letter part of the grade."

	aStream nextPut: self letter.

	"No determine which modifier to print if any at all."

	( modifier = #minus )

	ifTrue: [ aStream nextPut: $- ]

	ifFalse: [ 

		( modifier = #plus )

		ifTrue: [ aStream nextPut: $+ ]

	].! !



!Grade methodsFor: 'actions'!



decrement

	"This subtracts one full letter grade from the original grade. Thus, if 	you decrement a C+, you get a D+.  But, since we are representing 	the grades as integers internally, we must actually INCREASE the 	ascii value."



	"First, test te see if we are decrementing a D. If we are, we must be 	sure to skip over E since there is no E in the grading system."

	( letter < 68 )

	ifTrue: [ letter := letter + 1 ]

	ifFalse: [

		( letter == 68 )

		ifTrue: [ letter := 70 ]

	]!



increment

	"This adds one full letter grade from the original grade. Thus, if 	you increment a C+, you get a B+.  But, since we are representing 	the grades as integers internally, we must actually DECREASE the 	ascii value."



	"First test to see if we are incrementing an F. If we are, we must be 	sure to skip over E since there is no E in the grading system."

	( letter == 70 )

	ifTrue: [ letter := letter - 2 ]

	ifFalse: [

		"Now we must also be careful not to increment an A."

		( letter > 65 )

		ifTrue: [ letter := letter - 1 ]

	]! !



!Grade methodsFor: 'initialization'!



letter: letterPart modifier: modifierPart

	"Initialize data."



	letter := letterPart.

	modifier := modifierPart.



	^self! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



Grade class

	instanceVariableNames: ''!





!Grade class methodsFor: 'instance creation'!



letter: letter modifier: modifier

	"Check to see if it's a legal letter grade. If it is, then convert to an 	integer and store it that way."



	( letter asInteger < 71 & letter asInteger > 64 )

	ifTrue: [ ^(self new) letter: letter asInteger modifier: modifier ]!



new: grade

	"Alternative way of instantiation...using a string."



	| letterP modifierP |

	letterP := grade at: 1.

	"Check to see if there is even a sign with the grade."

	( grade size > 1 )

	ifTrue: [

		modifierP := grade at: 2.

		"Check to see if it's a -"

		( modifierP = $- )

		ifTrue: [ modifierP := #minus ]

		ifFalse: [

			"Check to see if it's a +"

			( modifierP = $+ )

			ifTrue: [modifierP := #plus ]

		]

	]

	ifFalse: [ modifierP := #none ].



	^self letter: letterP modifier: modifierP! !



