Object subclass: #Boggle
	instanceVariableNames: 'guessedList board correctGuesses dictionary '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Assignment2'!
Boggle comment:
'This Instance controls the operations of the Boggle game, it initializes all values needed for the game and handles wordGuesses and correctGuesses queries.  It holds the main game loop with the timer, and is responsible for knowing when the game is over, and what to do when the game is over.

Instance Variables:

guessedList	<OrderedCollection>	holds a list of all of the players guesses which will be looked at later to check how many of them belong in the board.  It only holds guesses that are present in the dictionary.
board	<BoggleGrid>	Element of class BoggleGrid which holds information about the game board
correctGuesses	<SmallInteger>	holds the number of correctGuesses at the end of a game and is returned in method correctGuesses
dictionary	<Dictionary>	is loaded with all of the words in the dictionary file DictFileName'!


!Boggle methodsFor: 'printing'!

printOn: aStream



     board printOn: aStream.



^self.! !

!Boggle methodsFor: 'starting'!

playWithGrid: aString
	"starts playing the game with given string representing the board"
	
	self loadDictionary.

	correctGuesses:=0.

	guessedList := OrderedCollection new.

    	board := BoggleGrid withString: aString.

	[ (Delay forSeconds: 180) wait.		
		Transcript show: 'GAME OVER'  ; cr. 
		Transcript show: 'You had  '.
		correctGuesses:= (self getNumRight).
		Transcript print: correctGuesses.
		Transcript show: '   Correct Guesses' ; cr.
		] fork.


^self.! !

!Boggle methodsFor: 'private'!

getNumRight

	"traverses through all of the guesses to see if board contains them"
	|numRight|
	numRight := 0.

	[guessedList size > 0]whileTrue:[
		|tempString|
		tempString := guessedList removeFirst.
		(board contains: tempString)
		ifTrue:[
			numRight:= numRight+1.
		].
	].
	


^numRight.!

loadDictionary
	"loads the dictionary file into our dictionary"

	|dictFileName dictStream tempString|
	dictionary := Dictionary new.
	dictFileName := DictFileName asFilename.
	dictStream := dictFileName readStream.
	[dictStream atEnd] whileFalse:[
		tempString := dictStream upTo: (Character cr).
		dictionary at: tempString put: tempString.
	].

^self.! !

!Boggle methodsFor: 'playing'!

correctGuesses



^correctGuesses.!

wordGuess: aString



	(guessedList includes: aString)
	ifFalse:[
		(dictionary includes: aString)
		ifTrue:[
			guessedList add: aString.
			^true.
		].
	].	

^false.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Boggle class
	instanceVariableNames: ''!


!Boggle class methodsFor: 'instance creation'!

start

    |rStream tempString position aValue|

    position :=1.

    tempString := String new: 16.

    rStream := Random new.

    aValue := $a asInteger.

    [position<17] whileTrue:[



         tempString at: position put: ((((rStream next)*26.0) asInteger +aValue)asCharacter).

         position:=position+1.

    ].    



^self new playWithGrid: tempString.!

startWithGrid: aString

	

^self new playWithGrid: aString.! !

Object subclass: #BoggleGrid
	instanceVariableNames: 'gridArray '
	classVariableNames: 'Directions '
	poolDictionaries: ''
	category: 'Assignment2'!
BoggleGrid comment:
'THIS is the basic board for the boggle game, it is represented as a single dimensional array internally but can be accessed as if it were a two dimensional array using modular arithmetic.  The class uses a Depth First Search to know if it contains a given word.

Instance Variables:

gridArray	<ByteArray>	 the array containing all of the characters on the board.

Class Variables:

Directions	<Array> an array of all of the directions you can travel in a search across the board.'!


!BoggleGrid methodsFor: 'printing'!

printOn: aStream
		
	"prints the grid in 4x4 format on the screen"
	|tempString loop1 loop2|
	tempString := String new: 8.
	loop1 := 1.
      aStream nextPut: Character cr .
	[loop1<5] whileTrue:[
		loop2:=1.
		[loop2<5] whileTrue:[
			tempString at: (2*loop2-1) put: $ .
			tempString at: (2*loop2) put: (self atX: loop2 Y: loop1).
			loop2 := loop2 + 1.
		].
		aStream nextPutAll: tempString.
		aStream nextPut: Character cr .
		loop1 := loop1+1.
	].
^self.! !

!BoggleGrid methodsFor: 'private'!

isValidBogglePoint: aPoint
	"returns true if the points domain and range are within the 4x4 values of the Boggle grid."


	((aPoint x <=4)&(aPoint x>=1)&(aPoint y<=4)&(aPoint y>=1))	
	ifTrue:[
		^true.
	].
^false.!

toPoint: index
	"for private use converts an index like 3 to a point like 3#1"

	|newPoint xv|
	xv:= (index - ((index quo: 4)* 4)).
	(xv=0)ifTrue:[
		xv:=4.
	].
	newPoint := Point x: xv y: ((((index-1) / 4)asInteger)+1). 


^newPoint! !

!BoggleGrid methodsFor: 'accessing'!

atPoint: aPoint
	"returns the value as a byte at aPoint"

	|index|

	index := (aPoint y)-1*4+(aPoint x).

	(index<1 | index >16)

	ifTrue:[

		Error raiseSignal: 'invalid grid location'.

	].

^gridArray at: index.!

atX: x Y: y
	"returns the value  as a byte in the board at the coordinates x y."
	|index|

	index := y-1*4+x.

	(index<1 | index >16)

	ifTrue:[

		Error raiseSignal: 'invalid grid location'.

	].

^gridArray at: index.!

contains: aString
	"returns true if the board contains aString, according to standard boggle rules"
	|loop list dir tempList counter length|
 	list := SortedCollection new.		
	dir := BoggleGrid directions.
	  
	"first find the every grid location containing the first character in the string"
	loop:=1.
	[loop < 17] whileTrue: [
		(((gridArray at: loop)asCharacter) = (aString at: 1) ) 
		ifTrue:[
			list add: (self toPoint: loop).				
 		].	
		loop:=loop+1.
	].

	length := (aString size).
	counter := 2.

	"use a breadth first search"	
	[counter <= length]whileTrue:[
		
		tempList := SortedCollection new: 10. 
		[list size = 0] whileFalse:[	
			|tempPoint oldPoint|
			loop:=1.
			oldPoint := list removeFirst.
			[loop<9]whileTrue:[
				tempPoint := oldPoint + (dir at: loop).
				(self isValidBogglePoint: tempPoint)
				ifTrue:[
			 		((self atPoint:tempPoint) asCharacter  = (aString at: counter))
					ifTrue:[
						tempList add: tempPoint.	
					].
				].
				loop := loop +1.	
			].
		].
		counter := counter + 1.
		list := tempList.
	].
	
^(list size = 0) not.!

setWithString: aString

	gridArray := aString. "asByteArray."

^self! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BoggleGrid class
	instanceVariableNames: ''!


!BoggleGrid class methodsFor: 'instance creation'!

withString: aString

	BoggleGrid initialize.  "makes sure that class variables are initialized"

^self new setWithString: aString.! !

!BoggleGrid class methodsFor: 'initialization'!

initialize
	|p|
	Directions := Array new: 8.
	p:= Point x: 1 y: 1.
	Directions at: 1 put: p.
	p:= Point x: 1 y: 0.
	Directions at: 2 put: p.
	p:= Point x: 1 y: -1.
	Directions at: 3 put: p.
	p:= Point x: 0 y: 1.
	Directions at: 4 put: p.
	p:= Point x: 0 y: -1.
	Directions at: 5 put: p.
	p:= Point x: -1 y: 1.
	Directions at: 6 put: p.
	p:= Point x: -1 y: 0.
	Directions at: 7 put: p.
	p:= Point x: -1 y: -1.
	Directions at: 8 put: p.
^self.! !

!BoggleGrid class methodsFor: 'general inquiries'!

directions


^Directions! !
BoggleGrid initialize!


