Object subclass: #Boggle
	instanceVariableNames: 'board correctGuesses dictionary guessList '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Assignment3'!
Boggle comment:
'Class Boggle:

Lets the user play a game of Boggle, using an implementation of the BoggleGrid class.  The user starts the game by creating an instance of this class, and then guesses words he/she thinks are in the grid.



Instance Variables:



board	<BoggleGrid>	contains the board/grid that is currently being used

correctGuesses	<Boggle>	number of correct guesses (words in grid that the user guessed)

dictionary	<Dictionary>	holds the dictionary that is from a file on disk

guessList	<Boggle>	holds the list of words that the user guesses'!


!Boggle methodsFor: 'accessing'!

getLetter: pt
	^(board letterAtPoint: pt).! !

!Boggle methodsFor: 'playing'!

correctGuesses

	" Returns the number of correct guesses the user had "



^correctGuesses.!

wordGuess: aString
	" Lets the user guess a word that may be in the Boggle Grid.  First tests if word is already in the list of guesses.  If not, it tests if the word is in the dictionary.  If it is, the word is added to the list of guesses. If not, do nothing-- since it can't be a correct guess. "


	(guessList includes: aString)
		ifFalse: [(dictionary includes: aString)
					ifTrue: [ guessList add: aString.
							^true.
					].
				].	
	^false.! !

!Boggle methodsFor: 'private'!

getCorrectGuesses

	" Finds and returns the number of correct guesses by cycling through list of guesses and testing to see if each one is in the board and in the dictionary "

	|correct tempString|

	correct := 0.
	[guessList size > 0] whileTrue:[
		tempString := guessList removeFirst.
		(board contains: tempString)
			ifTrue:[ correct := correct + 1. ].
	].
	correctGuesses := correct.

^correctGuesses.!

loadDictionary
	" Loads the dictionary to be used for searching for correct Boggle Words "

	|dictFilename dictStream tempString|

	dictionary := Dictionary new.
	dictFilename := 'dict' asFilename.
	dictStream := dictFilename readStream.

	[dictStream atEnd] whileFalse:[
		tempString := dictStream upTo: (Character cr).
		dictionary at: tempString put: tempString.
	].

^self.! !

!Boggle methodsFor: 'initialization'!

useGrid: aString
	" Start Boggle with a specific grid "


	self loadDictionary.
	correctGuesses := 0.
	guessList := OrderedCollection new.
    	board := BoggleGrid useString: aString.

	[ (Delay forSeconds: 90) wait.		
		"Transcript show: 'GAME OVER'; cr. 
		Transcript show: 'You had '.
		Transcript print: (self getCorrectGuesses).
		Transcript show: ' correct guesses'; cr."
	] fork.

^self.! !

!Boggle methodsFor: 'printing'!

printOn: aStream

     

	board printOn: aStream.

	

^self.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Boggle class
	instanceVariableNames: ''!


!Boggle class methodsFor: 'instance creation'!

start

	" Create a random string of lowercase letters, and start the game with that as the grid "



    | index randomStream tempString aValue|



    index :=1.

    tempString := String new: 16.

    randomStream := Random new.

    aValue := $a asInteger.

    [index<17] whileTrue:[

         tempString at: index put: (   (  ((randomStream next)*26.0)asInteger + aValue  )asCharacter   ).

         index := index+1.

    ].    



^self new useGrid: tempString.!

startWithGrid: aString

	" Start the grid with the string the user inputs "

	

^self new useGrid: aString.! !

!Boggle class methodsFor: 'class initialization'!

initialize



"simple initialization"







| dictStream counter|



counter:= 0.



dictFileName:= 'dict' asFilename.



dictStream:= dictFileName readStream.







dictionary:= Set new.







[counter < (dictionary size)] whileTrue: [



	dictionary add: [dictStream upTo: (Character cr)].



	dictStream next.



	counter:= counter + 1.]! !

ControllerWithMenu subclass: #BoggleTimerController
	instanceVariableNames: 'count status done '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Assignment3'!


!BoggleTimerController methodsFor: 'private'!

startTimer



"Starts the timer for aSec seconds"







	count:=0.



[count <= 8] whileTrue: [



count:= count + 1.







	[ (Delay forSeconds: 1) wait.



	self model decrement: count.



		 ] fork.







].!

startTimer: int



"Starts the timer for aSec seconds"







done:=true.



count:=int.



status:=false.



count >= 95 ifTrue: [^true].







	[ (Delay forSeconds: 1) wait.



	count:=count +3.



	self model decrement: count.



	self startTimer: 0] fork.! !

!BoggleTimerController methodsFor: 'control'!

controlActivity



"Starts the timer for aSec seconds"



status:= self model status.



(status = true) & (done = false) ifTrue: [self startTimer: 0].!

controlInitialize



"initialization of control"



super controlInitialize!

isControlWanted











^true.! !

!BoggleTimerController methodsFor: 'initialize-release'!

initialize







count:= 0.



status:=false.



done:=false.



super initialize.! !

Object subclass: #BoggleGrid
	instanceVariableNames: 'gridArray '
	classVariableNames: 'Directions '
	poolDictionaries: ''
	category: 'Assignment3'!
BoggleGrid comment:
'Class BoggleGrid:

Creates grid for use in the game of Boggle.  It has the functionality to test for words in the grid, etc.

The most confusing part is the search function, which cycles through the guessed words, and letter by letter sees if they can be found in the grid array using a distinct path.  If so, then the guess is correct (as long as the word is also in the dictionary), if not, the guess is not correct and not counted.





Instance Variables:



gridArray	<String>	Holds the grid in a string/array of characters



Class Variables:



Directions	<Array>		Holds an array of points, which are the directions from the origin that the search 							function can move to find the next possible letter of a string (used to see if a 							guessed word is in the grid)'!


!BoggleGrid methodsFor: 'accessing'!

atPoint: aPoint

	" Index the point on the grid (1-16), and return the value at that index "



	|index|



	index := (aPoint y) - 1 * 4 + (aPoint x).

	(index<1 | index >16)

		ifTrue:[ Error raiseSignal: 'invalid grid location'. ].



^gridArray at: index.!

atX: x Y: y

	" Just like atPoint:, except this directly indexes the x,y coordinates and returns the value at

		that index. "



	|index|



	index := y - 1 * 4 + x.

	(index<1 | index >16)

		ifTrue:[ Error raiseSignal: 'invalid grid location'. ].



^gridArray at: index.!

contains: aString

	" Figure out if the guessed string is in the grid "



	|loop aPointList aDir tempList counter length bPoint aPoint|

 	

	aPointList := SortedCollection new.		

	aDir := BoggleGrid directions.

	length := (aString size).

	counter := 2.

	loop := 1.

" Loop through grid and if the letter at the particular grid index is equal to the first letter of the guessed word,

	save it in aPointList (as a point, not an index) "

	

	[loop < 17] whileTrue: [

		(((gridArray at: loop)asCharacter) = (aString at: 1) ) 

			ifTrue:[ aPointList add: (self toPoint: loop).	].	

		loop:=loop+1.

	].



" Cycle through the guessed word, and scan the grid for a valid path that outlines that particular word "

	

	[counter <= length]whileTrue:[

		tempList := SortedCollection new: 10. 

		[aPointList size = 0] whileFalse:[	

			aPoint := aPointList removeFirst.

			loop := 1.

			[loop < 9] whileTrue:[

				bPoint := aPoint + (aDir at: loop).

				(self isValidGridPoint: bPoint)

				ifTrue:[ ((self atPoint:bPoint) asCharacter  = (aString at: counter))

							ifTrue:[ tempList add: bPoint.	].

				].

				loop := loop +1.	

			].

		].

		counter := counter + 1.

		aPointList := tempList.

	].

	

^(aPointList size = 0) not.!

letterAtPoint: p
	"return letter at given grid index"


	^(gridArray at: p).!

setString: aString

	" Called by useString, this layer actually does the setting of the string "



	gridArray := aString.



^self! !

!BoggleGrid methodsFor: 'private'!

isValidGridPoint: aPoint

	" Determine if the point is within the bounds of the grid "



	( (aPoint x>=1) & (aPoint y>=1) & (aPoint x<=4) & (aPoint y<=4))	

		ifTrue:[ ^true. ]

		ifFalse: [ ^false. ].!

toPoint: index

	" Convert a grid index back into a grid point "

	

	|aPoint x1 y1|

	

	x1 := (index rem: 4).

	y1 := ( (((index-1)/4)asInteger) + 1 ).

	(x1=0)

		ifTrue:[ x1 := 4.].

	aPoint := Point x: x1 y: y1. 



^aPoint! !

!BoggleGrid methodsFor: 'printing'!

printOn: aStream

	" Print the Boggle Grid in a nice 4x4 block, with one space between each letter. "



	|tempString loop1 loop2|



	tempString := String new: 8.

	loop1 := 1.

      aStream nextPut: Character cr .

	[loop1 < 5] whileTrue:[

		loop2 := 1.

		[loop2 < 5] whileTrue:[

				tempString at: (2*loop2) put: $ .

				tempString at: (2*loop2-1) put: (self atX: loop2 Y: loop1).

				loop2 := loop2 + 1.

		].

		aStream nextPutAll: tempString.

		aStream nextPut: Character cr .

		loop1 := loop1+1.

	].

^self.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BoggleGrid class
	instanceVariableNames: ''!


!BoggleGrid class methodsFor: 'external accessing'!

directions

	" duh "



^Directions! !

!BoggleGrid class methodsFor: 'initialization'!

initialize

	

	|p|

	Directions := Array new: 8.

	p:= Point x: 1 y: 1.

	Directions at: 1 put: p.

	p:= Point x: 1 y: 0.

	Directions at: 2 put: p.

	p:= Point x: 1 y: -1.

	Directions at: 3 put: p.

	p:= Point x: 0 y: 1.

	Directions at: 4 put: p.

	p:= Point x: 0 y: -1.

	Directions at: 5 put: p.

	p:= Point x: -1 y: 1.

	Directions at: 6 put: p.

	p:= Point x: -1 y: 0.

	Directions at: 7 put: p.

	p:= Point x: -1 y: -1.

	Directions at: 8 put: p.

^self.! !

!BoggleGrid class methodsFor: 'instance creation'!

useString: aString

	" Sets the string to be used for the grid "

	

	BoggleGrid initialize.



^self new setString: aString.! !

ApplicationModel subclass: #BoggleApplication
	instanceVariableNames: 'timer timermodel boggleGame guessword word guesslist console p1 p2 p3 p4 p5 p6 p7 p8 p9 p10 p11 p12 p13 p14 p15 p16 '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Assignment3'!


!BoggleApplication methodsFor: 'initialize-release'!

initialize
	"initialize appropriate values, and start the game"

	super initialize.
	
	boggleGame := Boggle start.

	self initializeConsole.
	self initializeGrid.
	self initializeGuesslist.
	self initializeGuessword.

	timer := BoggleTimerView new.
	timermodel := BoggleTimerModel new.
	timer model: timermodel.

	timermodel status: true.
	self startTimer.

	self changedBoggleTimer.!

initializeConsole

	console := ValueHolder newString.!

initializeGrid

	p1 := (boggleGame getLetter: 1) asSymbol.
	p2 := (boggleGame getLetter: 2) asSymbol.
	p3 := (boggleGame getLetter: 3) asSymbol.
	p4 := (boggleGame getLetter: 4) asSymbol.
	p5 := (boggleGame getLetter: 5) asSymbol.
	p6 := (boggleGame getLetter: 6) asSymbol.
	p7 := (boggleGame getLetter: 7) asSymbol.
	p8 := (boggleGame getLetter: 8) asSymbol.
	p9 := (boggleGame getLetter: 9) asSymbol.
	p10 := (boggleGame getLetter: 10) asSymbol.
	p11 := (boggleGame getLetter: 11) asSymbol.
	p12 := (boggleGame getLetter: 12) asSymbol.
	p13 := (boggleGame getLetter: 13) asSymbol.
	p14 := (boggleGame getLetter: 14) asSymbol.
	p15 := (boggleGame getLetter: 15) asSymbol.
	p16 := (boggleGame getLetter: 16) asSymbol.!

initializeGuesslist

	guesslist := SelectionInList new.!

initializeGuessword

	guessword := ValueHolder newString.
	guessword onChangeSend: #changedGuessword to: self.! !

!BoggleApplication methodsFor: 'aspects'!

console
	^console!

guesslist

	^guesslist.!

guessword

	^guessword.!

p1
^p1!

p10
^p10!

p11
^p11!

p12
^p12!

p13
^p13!

p14
^p14!

p15
^p15!

p16
^p16!

p2
^p2!

p3
^p3!

p4
^p4!

p5
^p5!

p6
^p6!

p7
^p7!

p8
^p8!

p9
^p9!

timer

^timer! !

!BoggleApplication methodsFor: 'actions'!

makeguess
	| list selectionsInList |

	boggleGame wordGuess: word.
	
	selectionsInList := self guesslist.
	list := selectionsInList list.
	(list includes: word) ifFalse: [list add: word].

	self console value: 'You made a guess!!'.! !

!BoggleApplication methodsFor: 'change messages'!

changedBoggleTimer

	timer update: #value.!

changedGuessword
	| temp |

	temp := self guessword value.
	word := temp.! !

!BoggleApplication methodsFor: 'private'!

startTimer
	| correct |


	[ (Delay forSeconds: 90) wait.
		correct := boggleGame getCorrectGuesses.
		self console value: 'GAME OVER. Correct Guesses: ', correct printString.
		 ] fork.!

startTimer: aSec
	| count |

	count:=0.

	[count <= 90] whileTrue: [
		[ (Delay forSeconds: 1) wait.
			timermodel decrement ;cr
		]fork. 

	self changedBoggleTimer.
	count:= count + 1.
	].!

status

	^true.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BoggleApplication class
	instanceVariableNames: ''!


!BoggleApplication class methodsFor: 'interface specs'!

windowSpec

	"UIPainter new openOnClass: self andSelector: #windowSpec"



	<resource: #canvas>

	^#(#FullSpec 

		#window: 

		#(#WindowSpec 

			#label: 'Boggle with GUI' 

			#min: #(#Point 489 267 ) 

			#max: #(#Point 489 267 ) 

			#bounds: #(#Rectangle 432 260 921 527 ) 

			#colors: 

			#(#LookPreferences 

				#setForegroundColor: #(#ColorValue #black ) 

				#setBackgroundColor: #(#ColorValue 1024 8191 1024 ) 

				#setSelectionForegroundColor: #(#ColorValue #black ) 

				#setSelectionBackgroundColor: #(#ColorValue 4915 4915 4915 ) 

				#setBorderColor: #(#ColorValue #black ) ) 

			#isEventDriven: true ) 

		#component: 

		#(#SpecCollection 

			#collection: #(

				#(#ActionButtonSpec 

					#layout: #(#Rectangle 391 98 450 133 ) 

					#colors: 

					#(#LookPreferences 

						#setBackgroundColor: #(#ColorValue 1024 1024 8191 ) ) 

					#model: #makeguess 

					#label: 'Guess' 

					#defaultable: true ) 

				#(#DividerSpec 

					#layout: #(#Rectangle 1 199 486 203 ) 

					#colors: 

					#(#LookPreferences 

						#setForegroundColor: #(#ColorValue #blue ) 

						#setBackgroundColor: #(#ColorValue #blue ) 

						#setSelectionForegroundColor: #(#ColorValue #blue ) 

						#setSelectionBackgroundColor: #(#ColorValue #blue ) ) ) 

				#(#DividerSpec 

					#layout: #(#Rectangle 354 2 454 199 ) 

					#colors: 

					#(#LookPreferences 

						#setForegroundColor: #(#ColorValue #blue ) 

						#setBackgroundColor: #(#ColorValue #blue ) 

						#setSelectionForegroundColor: #(#ColorValue #blue ) 

						#setSelectionBackgroundColor: #(#ColorValue #blue ) ) 

					#orientation: #vertical ) 

				#(#LabelSpec 

					#layout: #(#Point 17 3 ) 

					#label: 'Guessed Words:' ) 

				#(#LabelSpec 

					#layout: #(#Point 357 52 ) 

					#label: 'Enter Word to Guess:' ) 

				#(#DividerSpec 

					#layout: #(#Rectangle 132 2 232 199 ) 

					#colors: 

					#(#LookPreferences 

						#setForegroundColor: #(#ColorValue #blue ) 

						#setBackgroundColor: #(#ColorValue #blue ) 

						#setSelectionForegroundColor: #(#ColorValue #blue ) 

						#setSelectionBackgroundColor: #(#ColorValue #blue ) ) 

					#orientation: #vertical ) 

				#(#LabelSpec 

					#layout: #(#Point 206 33 ) 

					#isOpaque: true 

					#label: '<=GRID=>' 

					#style: #large ) 

				#(#LabelSpec 

					#layout: #(#Point 208 4 ) 

					#label: '!!BOGGLE!!' 

					#style: #large ) 

				#(#LabelSpec 

					#layout: #(#Point 179 181 ) 

					#label: 'Created by Eric Musgrave' 

					#style: #small ) 

				#(#DividerSpec 

					#layout: #(#Rectangle 133 29 354 33 ) 

					#colors: 

					#(#LookPreferences 

						#setBackgroundColor: #(#ColorValue #blue ) ) ) 

				#(#DividerSpec 

					#layout: #(#Rectangle 135 180 354 184 ) ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 360 73 485 99 ) 

					#colors: 

					#(#LookPreferences 

						#setBackgroundColor: #(#ColorValue 1024 8191 8191 ) ) 

					#model: #guessword 

					#tabable: false 

					#style: #large ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 201 61 221 86 ) 

					#model: #p1 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 223 61 243 86 ) 

					#model: #p2 

					#alignment: #center 

					#style: #large 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 201 89 221 114 ) 

					#model: #p5 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 223 89 243 114 ) 

					#model: #p6 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 245 88 265 113 ) 

					#model: #p7 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 267 88 287 113 ) 

					#model: #p8 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 201 116 221 141 ) 

					#model: #p9 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 223 116 243 141 ) 

					#model: #p10 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 245 116 265 141 ) 

					#model: #p11 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 267 116 287 141 ) 

					#model: #p12 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 201 143 221 168 ) 

					#model: #p13 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 223 142 243 167 ) 

					#model: #p14 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 245 142 265 167 ) 

					#model: #p15 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 266 143 286 168 ) 

					#model: #p16 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 246 60 266 85 ) 

					#model: #p3 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true 

					#type: #text ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 268 60 288 85 ) 

					#model: #p4 

					#tabable: false 

					#alignment: #center 

					#style: #large 

					#isReadOnly: true ) 

				#(#GroupBoxSpec 

					#layout: #(#Rectangle 196 56 292 172 ) 

					#colors: 

					#(#LookPreferences 

						#setBackgroundColor: #(#ColorValue 0 3317 0 ) 

						#setSelectionBackgroundColor: #(#ColorValue 0 3317 0 ) ) 

					#style: #large ) 

				#(#RegionSpec 

					#layout: #(#Rectangle 138 2 348 26 ) 

					#colors: 

					#(#LookPreferences 

						#setForegroundColor: #(#ColorValue 0 0 5013 ) ) 

					#lineWidth: 2 

					#regionType: #ellipse ) 

				#(#RegionSpec 

					#layout: #(#Rectangle 329 2 354 26 ) 

					#isOpaque: true 

					#colors: 

					#(#LookPreferences 

						#setForegroundColor: #(#ColorValue #black ) 

						#setBackgroundColor: #(#ColorValue #cyan ) ) 

					#lineWidth: 4 

					#regionType: #ellipse ) 

				#(#RegionSpec 

					#layout: #(#Rectangle 133 3 158 27 ) 

					#isOpaque: true 

					#colors: 

					#(#LookPreferences 

						#setForegroundColor: #(#ColorValue #black ) 

						#setBackgroundColor: #(#ColorValue #cyan ) ) 

					#lineWidth: 4 

					#regionType: #ellipse ) 

				#(#LabelSpec 

					#layout: #(#Point 95 202 ) 

					#colors: 

					#(#LookPreferences 

						#setForegroundColor: #(#ColorValue 1261 1261 1261 ) 

						#setBackgroundColor: #(#ColorValue 1261 1261 1261 ) ) 

					#label: 'Console:' ) 

				#(#InputFieldSpec 

					#layout: #(#Rectangle 6 222 242 258 ) 

					#colors: 

					#(#LookPreferences 

						#setForegroundColor: #(#ColorValue #blue ) 

						#setBackgroundColor: #(#ColorValue #lightCyan ) ) 

					#model: #console 

					#tabable: false 

					#alignment: #center 

					#style: #systemDefault 

					#isReadOnly: true 

					#type: #text ) 

				#(#SequenceViewSpec 

					#layout: #(#Rectangle 9 28 125 189 ) 

					#name: #guesslist 

					#colors: 

					#(#LookPreferences 

						#setForegroundColor: #(#ColorValue #blue ) 

						#setBackgroundColor: #(#ColorValue #paleGreen ) ) 

					#model: #guesslist 

					#useModifierKeys: true 

					#selectionType: #highlight ) 

				#(#LabelSpec 

					#layout: #(#Point 333 202 ) 

					#label: 'Timer:' ) 

				#(#ArbitraryComponentSpec 

					#layout: #(#Rectangle 249 222 485 258 ) 

					#name: #timer 

					#colors: 

					#(#LookPreferences 

						#setBackgroundColor: #(#ColorValue 1024 8191 8191 ) ) 

					#component: #timer ) ) ) )! !

Object subclass: #BoggleTimerModel
	instanceVariableNames: 'time status '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Assignment3'!


!BoggleTimerModel methodsFor: 'actions'!

decrement



"test method"



time:= 50.



self changed: #value.!

decrement: int



"decrements the counter for the ball drop view"



time:= time + int.



self changed: #value.! !

!BoggleTimerModel methodsFor: 'accessing'!

status



^status!

status: aBool



status:= aBool.



self changed: #value.!

time



^time.!

time: anInt



time:= anInt.! !

!BoggleTimerModel methodsFor: 'initialize-release'!

initialize







	time := 20.



	status:=false.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BoggleTimerModel class
	instanceVariableNames: ''!


!BoggleTimerModel class methodsFor: 'instance creation'!

new







^super new initialize.! !

View subclass: #BoggleTimerView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Assignment3'!


!BoggleTimerView methodsFor: 'controller accessing'!

defaultControllerClass

	^BoggleTimerController! !

!BoggleTimerView methodsFor: 'updating'!

update: anAspect



anAspect == #value

	ifTrue: [self invalidate].! !

!BoggleTimerView methodsFor: 'accessing'!

model: aModel



	super model: aModel.



	self invalidate.! !

!BoggleTimerView methodsFor: 'displaying'!

displayOn: aGraphicsContext

	"create circle"



	| gc circle x |



	x := self model time.

	self model isNil ifTrue: [^self].

	gc := aGraphicsContext.

	circle := Circle center: x@10 radius: 10.

	circle asFiller displayOn: (gc paint: ColorValue blue).! !
Boggle initialize!

BoggleGrid initialize!


