Object subclass: #BoggleGrid
	instanceVariableNames: 'gridArray '
	classVariableNames: 'Directions '
	poolDictionaries: ''
	category: 'Assignment2'!
BoggleGrid comment:
'Class BoggleGrid:

Creates grid for use in the game of Boggle.  It has the functionality to test for words in the grid, etc.

The most confusing part is the search function, which cycles through the guessed words, and letter by letter sees if they can be found in the grid array using a distinct path.  If so, then the guess is correct (as long as the word is also in the dictionary), if not, the guess is not correct and not counted.





Instance Variables:



gridArray	<String>	Holds the grid in a string/array of characters



Class Variables:



Directions	<Array>		Holds an array of points, which are the directions from the origin that the search 							function can move to find the next possible letter of a string (used to see if a 							guessed word is in the grid)'!


!BoggleGrid methodsFor: 'accessing'!

atPoint: aPoint

	" Index the point on the grid (1-16), and return the value at that index "



	|index|



	index := (aPoint y) - 1 * 4 + (aPoint x).

	(index<1 | index >16)

		ifTrue:[ Error raiseSignal: 'invalid grid location'. ].



^gridArray at: index.!

atX: x Y: y

	" Just like atPoint:, except this directly indexes the x,y coordinates and returns the value at

		that index. "



	|index|



	index := y - 1 * 4 + x.

	(index<1 | index >16)

		ifTrue:[ Error raiseSignal: 'invalid grid location'. ].



^gridArray at: index.!

contains: aString

	" Figure out if the guessed string is in the grid "



	|loop aPointList aDir tempList counter length bPoint aPoint|

 	

	aPointList := SortedCollection new.		

	aDir := BoggleGrid directions.

	length := (aString size).

	counter := 2.

	loop := 1.

" Loop through grid and if the letter at the particular grid index is equal to the first letter of the guessed word,

	save it in aPointList (as a point, not an index) "

	

	[loop < 17] whileTrue: [

		(((gridArray at: loop)asCharacter) = (aString at: 1) ) 

			ifTrue:[ aPointList add: (self toPoint: loop).	].	

		loop:=loop+1.

	].



" Cycle through the guessed word, and scan the grid for a valid path that outlines that particular word "

	

	[counter <= length]whileTrue:[

		tempList := SortedCollection new: 10. 

		[aPointList size = 0] whileFalse:[	

			aPoint := aPointList removeFirst.

			loop := 1.

			[loop < 9] whileTrue:[

				bPoint := aPoint + (aDir at: loop).

				(self isValidGridPoint: bPoint)

				ifTrue:[ ((self atPoint:bPoint) asCharacter  = (aString at: counter))

							ifTrue:[ tempList add: bPoint.	].

				].

				loop := loop +1.	

			].

		].

		counter := counter + 1.

		aPointList := tempList.

	].

	

^(aPointList size = 0) not.!

letterAtPoint: p
	"return letter at given grid index"


	^(gridArray at: p).!

setString: aString

	" Called by useString, this layer actually does the setting of the string "



	gridArray := aString.



^self! !

!BoggleGrid methodsFor: 'private'!

isValidGridPoint: aPoint

	" Determine if the point is within the bounds of the grid "



	( (aPoint x>=1) & (aPoint y>=1) & (aPoint x<=4) & (aPoint y<=4))	

		ifTrue:[ ^true. ]

		ifFalse: [ ^false. ].!

toPoint: index

	" Convert a grid index back into a grid point "

	

	|aPoint x1 y1|

	

	x1 := (index rem: 4).

	y1 := ( (((index-1)/4)asInteger) + 1 ).

	(x1=0)

		ifTrue:[ x1 := 4.].

	aPoint := Point x: x1 y: y1. 



^aPoint! !

!BoggleGrid methodsFor: 'printing'!

printOn: aStream

	" Print the Boggle Grid in a nice 4x4 block, with one space between each letter. "



	|tempString loop1 loop2|



	tempString := String new: 8.

	loop1 := 1.

      aStream nextPut: Character cr .

	[loop1 < 5] whileTrue:[

		loop2 := 1.

		[loop2 < 5] whileTrue:[

				tempString at: (2*loop2) put: $ .

				tempString at: (2*loop2-1) put: (self atX: loop2 Y: loop1).

				loop2 := loop2 + 1.

		].

		aStream nextPutAll: tempString.

		aStream nextPut: Character cr .

		loop1 := loop1+1.

	].

^self.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BoggleGrid class
	instanceVariableNames: ''!


!BoggleGrid class methodsFor: 'external accessing'!

directions

	" duh "



^Directions! !

!BoggleGrid class methodsFor: 'initialization'!

initialize

	

	|p|

	Directions := Array new: 8.

	p:= Point x: 1 y: 1.

	Directions at: 1 put: p.

	p:= Point x: 1 y: 0.

	Directions at: 2 put: p.

	p:= Point x: 1 y: -1.

	Directions at: 3 put: p.

	p:= Point x: 0 y: 1.

	Directions at: 4 put: p.

	p:= Point x: 0 y: -1.

	Directions at: 5 put: p.

	p:= Point x: -1 y: 1.

	Directions at: 6 put: p.

	p:= Point x: -1 y: 0.

	Directions at: 7 put: p.

	p:= Point x: -1 y: -1.

	Directions at: 8 put: p.

^self.! !

!BoggleGrid class methodsFor: 'instance creation'!

useString: aString

	" Sets the string to be used for the grid "

	

	BoggleGrid initialize.



^self new setString: aString.! !

Object subclass: #Boggle
	instanceVariableNames: 'board correctGuesses dictionary guessList '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Assignment2'!
Boggle comment:
'Class Boggle:

Lets the user play a game of Boggle, using an implementation of the BoggleGrid class.  The user starts the game by creating an instance of this class, and then guesses words he/she thinks are in the grid.



Instance Variables:



board	<BoggleGrid>	contains the board/grid that is currently being used

correctGuesses	<Boggle>	number of correct guesses (words in grid that the user guessed)

dictionary	<Dictionary>	holds the dictionary that is from a file on disk

guessList	<Boggle>	holds the list of words that the user guesses'!


!Boggle methodsFor: 'accessing'!

getLetter: pt
	^(board letterAtPoint: pt).! !

!Boggle methodsFor: 'playing'!

correctGuesses

	" Returns the number of correct guesses the user had "



^correctGuesses.!

wordGuess: aString

	" Lets the user guess a word that may be in the Boggle Grid.  First tests if word is already in the list of

		guesses.  If not, it tests if the word is in the dictionary.  If it is, the word is added to the list of guesses.

		If not, do nothing-- since it can't be a correct guess. "



	(guessList includes: aString)

		ifFalse: [(dictionary includes: aString)

					ifTrue: [ guessList add: aString.

							^true.

					].

				].	

	^false.! !

!Boggle methodsFor: 'private'!

getCorrectGuesses

	" Finds and returns the number of correct guesses by cycling through list of guesses and testing to see if

		each one is in the board and in the dictionary "



	|correct tempString|

	

	correct := 0.

	[guessList size > 0] whileTrue:[

		tempString := guessList removeFirst.

		(board contains: tempString)

			ifTrue:[ correct := correct + 1. ].

	].

	correctGuesses := correct.



^correctGuesses.!

loadDictionary

	" Loads the dictionary to be used for searching for correct Boggle Words "



	|dictFilename dictStream tempString|



	dictionary := Dictionary new.

	dictFilename := 'dict' asFilename.

	dictStream := dictFilename readStream.



	[dictStream atEnd] whileFalse:[

		tempString := dictStream upTo: (Character cr).

		dictionary at: tempString put: tempString.

	].



^self.! !

!Boggle methodsFor: 'initialization'!

useGrid: aString

	" Start Boggle with a specific grid "



	self loadDictionary.

	correctGuesses := 0.

	guessList := OrderedCollection new.

    	board := BoggleGrid useString: aString.



	[ (Delay forSeconds: 30) wait.		

		Transcript show: 'GAME OVER'; cr. 

		Transcript show: 'You had '.

		Transcript print: (self getCorrectGuesses).

		Transcript show: ' correct guesses'; cr.

	] fork.



^self.! !

!Boggle methodsFor: 'printing'!

printOn: aStream

     

	board printOn: aStream.

	

^self.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Boggle class
	instanceVariableNames: ''!


!Boggle class methodsFor: 'instance creation'!

start

	" Create a random string of lowercase letters, and start the game with that as the grid "



    | index randomStream tempString aValue|



    index :=1.

    tempString := String new: 16.

    randomStream := Random new.

    aValue := $a asInteger.

    [index<17] whileTrue:[

         tempString at: index put: (   (  ((randomStream next)*26.0)asInteger + aValue  )asCharacter   ).

         index := index+1.

    ].    



^self new useGrid: tempString.!

startWithGrid: aString

	" Start the grid with the string the user inputs "

	

^self new useGrid: aString.! !

ApplicationModel subclass: #BoggleApplication
	instanceVariableNames: 'updateTrigger boggleGame guessword guesslist console p1 p2 p3 p4 p5 p6 p7 p8 p9 p10 p11 p12 p13 p14 p15 p16 '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Assignment2'!


!BoggleApplication methodsFor: 'initialize-release'!

initialize
	super initialize.

	boggleGame := Boggle startWithGrid: 'starlightstaryou'.

	self initializeGuessword.
	self initializeGrid.!

initializeGrid
"	
p1 := String new asValue.
p2 := String new asValue.
p3 := String new asValue.
p4 := String new asValue.
p5 := String new asValue.
p6 := String new asValue.
p7 := String new asValue.
p8 := String new asValue.
p9 := String new asValue.
p10 := String new asValue.
p11 := String new asValue.
p12 := String new asValue.
p13 := String new asValue.
p14 := String new asValue.
p15 := String new asValue.
p16 := String new asValue.	"

	p1 := (boggleGame getLetter: 1) at: 2 asValue.
"	p2 := (boggleGame getLetter: 2)value.
	p3 := (boggleGame getLetter: 3)value.
	p4 := (boggleGame getLetter: 4)value.
	p5 := (boggleGame getLetter: 5)value.
	p6 := (boggleGame getLetter: 6)value.
	p7 := (boggleGame getLetter: 7)value.
	p8 := (boggleGame getLetter: 8)value.
	p9 := (boggleGame getLetter: 9)value.
	p10 := (boggleGame getLetter: 10)value.
	p11 := (boggleGame getLetter: 11)value.
	p12 := (boggleGame getLetter: 12)value.
	p13 := (boggleGame getLetter: 13)value.
	p14 := (boggleGame getLetter: 14)value.
	p15 := (boggleGame getLetter: 15)value.
	p16 := (boggleGame getLetter: 16)value."!

initializeGuessword

	guessword := ValueHolder newString.
	guessword onChangeSend: #changedName to: self.! !

!BoggleApplication methodsFor: 'aspects'!

guessword

	^guessword.!

p1
^p1!

p10
^p10!

p11
^p11!

p12
^p12!

p13
^p13!

p14
^p14!

p15
^p15!

p16
^p16!

p2
^p2!

p3
^p3!

p4
^p4!

p5
^p5!

p6
^p6!

p7
^p7!

p8
^p8!

p9
^p9! !

!BoggleApplication methodsFor: 'actions'!

makeguess

	boggleGame wordGuess: (guessword value).
	
	
	Transcript show: 'made guess'; cr.! !

!BoggleApplication methodsFor: 'change messages'!

changedName

	self makeguess.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BoggleApplication class
	instanceVariableNames: ''!


!BoggleApplication class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Boggle with GUI' 
			#min: #(#Point 489 248 ) 
			#max: #(#Point 489 248 ) 
			#bounds: #(#Rectangle 529 716 1018 964 ) 
			#colors: 
			#(#LookPreferences 
				#setForegroundColor: #(#ColorValue #black ) 
				#setBackgroundColor: #(#ColorValue 1024 8191 1024 ) 
				#setSelectionForegroundColor: #(#ColorValue #black ) 
				#setSelectionBackgroundColor: #(#ColorValue 4915 4915 4915 ) 
				#setBorderColor: #(#ColorValue #black ) ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 391 98 450 133 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 7544 1298 1298 ) ) 
					#model: #makeguess 
					#label: 'Guess' 
					#defaultable: true ) 
				#(#DividerSpec 
					#layout: #(#Rectangle 1 199 486 203 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #blue ) 
						#setBackgroundColor: #(#ColorValue #blue ) 
						#setSelectionForegroundColor: #(#ColorValue #blue ) 
						#setSelectionBackgroundColor: #(#ColorValue #blue ) ) ) 
				#(#DividerSpec 
					#layout: #(#Rectangle 354 2 454 199 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #blue ) 
						#setBackgroundColor: #(#ColorValue #blue ) 
						#setSelectionForegroundColor: #(#ColorValue #blue ) 
						#setSelectionBackgroundColor: #(#ColorValue #blue ) ) 
					#orientation: #vertical ) 
				#(#LabelSpec 
					#layout: #(#Point 17 3 ) 
					#label: 'Guessed Words:' ) 
				#(#LabelSpec 
					#layout: #(#Point 357 52 ) 
					#label: 'Enter Word to Guess:' ) 
				#(#DividerSpec 
					#layout: #(#Rectangle 132 2 232 199 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #blue ) 
						#setBackgroundColor: #(#ColorValue #blue ) 
						#setSelectionForegroundColor: #(#ColorValue #blue ) 
						#setSelectionBackgroundColor: #(#ColorValue #blue ) ) 
					#orientation: #vertical ) 
				#(#LabelSpec 
					#layout: #(#Point 206 33 ) 
					#isOpaque: true 
					#label: '<=GRID=>' 
					#style: #large ) 
				#(#LabelSpec 
					#layout: #(#Point 208 4 ) 
					#label: '!!BOGGLE!!' 
					#style: #large ) 
				#(#LabelSpec 
					#layout: #(#Point 179 181 ) 
					#label: 'Created by Eric Musgrave' 
					#style: #small ) 
				#(#DividerSpec 
					#layout: #(#Rectangle 133 29 354 33 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #blue ) ) ) 
				#(#DividerSpec 
					#layout: #(#Rectangle 135 180 354 184 ) ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 360 73 485 99 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 3072 8191 ) ) 
					#model: #guessword 
					#tabable: false 
					#style: #large ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 193 64 216 85 ) 
					#model: #p1 
					#type: #symbol ) ) ) )! !
BoggleGrid initialize!


